/*
	voclib.C

	VOC Library

	Version 1.0

	Copyright (C) 1994, Geoff Friesen B.Sc.
	All rights reserved.
*/

#pragma inline

#if !defined(__LARGE__)
#error Large Memory Model Expected
#endif

#include <alloc.H>
#include <conio.H>
#include <dir.H>
#include <dos.H>
#include <io.H>
#include <stdio.H>
#include <string.H>

#include "voclib.H"

static int error;
static int installed = 0;
static int voc_status;

static char *orig_voc_driver, *voc_driver;

/* ------------------------------------- */
/* int voc_Continue (void);              */
/*                                       */
/* Continue playback of a paused sample. */
/*                                       */
/* Return: ERROR or OK                   */
/* ------------------------------------- */

int voc_Continue (void)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   asm mov bx, 11
   asm call DWORD PTR voc_driver

   if (_AX)
   {
       error = VOCERR_CP_ERROR;
       return ERROR;
   }

   error = 0;
   return OK;
}

/* ------------------------------- */
/* int voc_DeInit (void);          */
/*                                 */
/* Deinitialize the sound blaster. */
/*                                 */
/* Return: ERROR or OK             */
/* ------------------------------- */

int voc_DeInit (void)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   /* Deinstall driver */

   asm mov bx, 9
   asm call DWORD PTR voc_driver

   farfree (orig_voc_driver);

   installed = 0;

   error = 0;

   return OK;
}

/* -------------------------------- */
/* int voc_Error (void);            */
/*                                  */
/* Retrieve most recent error code. */
/*                                  */
/* Return: error code               */
/* -------------------------------- */

int voc_Error (void)
{
   return error;
}

/* ---------------------------------------------- */
/* int voc_GetVersion (int *major, int *minor);   */
/*                                                */
/* Obtain the version number of the sound blaster */
/* driver.  The major portion is stored in *major */
/* and the minor portion in *minor.               */
/*                                                */
/* Return: ERROR or OK                            */
/* ---------------------------------------------- */

int voc_GetVersion (int *major, int *minor)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   asm mov bx, 0
   asm call DWORD PTR voc_driver

   asm les bx, major
   asm mov es:[bx], ah
   asm mov BYTE PTR es:[bx+1], 0

   asm les bx, minor
   asm mov es:[bx], al
   asm mov BYTE PTR es:[bx+1], 0

   error = 0;
   return OK;
}

/* ------------------------------------------------- */
/* int voc_Init (void);                              */
/*                                                   */
/* Load the sound blaster VOC driver (CT-VOICE.DRV), */
/* initialize the driver, and set the address of the */
/* status word.                                      */
/*                                                   */
/* Return: ERROR or OK                               */
/* ------------------------------------------------- */

int voc_Init (void)
{
   FILE *fp;
   char huge *filepointer;
   unsigned bytes_read, vout, voff, vseg;

   if (installed)
   {
       error = VOCERR_INSTALLED;
       return ERROR;
   }

   if ((fp = fopen ("CT-VOICE.DRV", "rb")) == NULL)
   {
       error = VOCERR_FILE_NOT_OPENED;
       return ERROR;
   }

   if ((orig_voc_driver = (char *) farmalloc (filelength (fileno(fp))+15L))
       == NULL)
   {
       (void) fclose (fp);
       error = VOCERR_INSUF_MEM;
       return ERROR;
   }

   voc_driver = MK_FP(FP_SEG(orig_voc_driver)+1, 0);

   filepointer = (char huge *) voc_driver;

   do
   {
	bytes_read = fread (filepointer, 1, 0x8000, fp);
	filepointer += bytes_read;
   }
   while (bytes_read == 0x8000);

   (void) fclose (fp);

   voff = FP_OFF(&voc_status);
   vseg = FP_SEG(&voc_status);

   asm mov bx, 3
   asm call DWORD PTR voc_driver
   asm mov vout, ax

   if (vout)
   {
       error = VOCERR_INIT_FAILED;
       return ERROR;
   }

   /* Set status word address */

   asm mov bx, 5
   asm mov es, vseg
   asm mov di, voff
   asm call DWORD PTR voc_driver

   installed = 1;

   error = 0;
   return OK;
}

/* ------------------------------- */
/* int voc_Pause (void);           */
/*                                 */
/* Pause the playback of a sample. */
/*                                 */
/* Return: ERROR or OK             */
/* ------------------------------- */

int voc_Pause (void)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   asm mov bx, 10
   asm call DWORD PTR voc_driver

   if (_AX)
   {
       error = VOCERR_CP_ERROR;
       return ERROR;
   }

   error = 0;
   return OK;
}

/* ---------------------------------------------------------- */
/* int voc_Play (char *filespec);                             */
/*                                                            */
/* Play a sample stored in a .VOC file specified by filespec. */
/* Play is aborted if a key is pressed.                       */
/*                                                            */
/* Return: ERROR or OK                                        */
/* ---------------------------------------------------------- */

int voc_Play (char *filespec)
{
   FILE *fp;
   int voff, vseg;
   char *song;
   char drive [MAXDRIVE];
   char dir [MAXDIR];
   char file [MAXFILE];
   char ext [MAXEXT];
   char path [MAXPATH];

   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   strcpy (path, filespec);

   if (!(fnsplit (path, drive, dir, file, ext) & EXTENSION))
   {
       strcpy (ext, ".VOC");
       fnmerge (path, drive, dir, file, ext);
   }

   if ((fp = fopen (path, "rb")) == NULL)
   {
       error = VOCERR_FILE_NOT_OPENED;
       return ERROR;
   }

   if ((song = (char *) farmalloc (filelength (fileno(fp)))) == NULL)
   {
       (void) fclose (fp);
       error = VOCERR_INSUF_MEM;
       return ERROR;
   }

   if (fread (song, filelength (fileno(fp)), 1, fp) != 1)
   {
       (void) fclose (fp);
       farfree (song);
       error = VOCERR_FILE_NOT_READ;
       return ERROR;
   }

   (void) fclose (fp);

   vseg = FP_SEG(song);
   voff = FP_OFF(song)+song [20];

   asm mov bx, 6
   asm mov es, vseg
   asm mov di, voff
   asm call DWORD PTR voc_driver

   while (voc_status)
      if (kbhit ())
      {
	  if (!getch ())
	      (void) getch ();

	  break;
      }

   asm mov bx, 8
   asm call DWORD PTR voc_driver

   farfree (song);

   error = 0;
   return OK;
}

/* -------------------------------------------------------- */
/* int voc_PlaySample (char *sample);                       */
/*                                                          */
/* Playback a sample stored in memory pointed to by sample. */
/* Sample must point to the first data block (skip header). */
/*                                                          */
/* Return: ERROR or OK                                      */
/* -------------------------------------------------------- */

int voc_PlaySample (char *sample)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   asm mov bx, 6
   asm les di, sample
   asm call DWORD PTR voc_driver

   error = 0;
   return OK;
}

/* ------------------------------------------- */
/* int voc_SetSpeaker (int onoff);             */
/*                                             */
/* Turn on the loudspeaker if onoff is nonzero */
/* otherwise turn off the loudspeaker.         */
/*                                             */
/* Return: ERROR or OK                         */
/* ------------------------------------------- */

int voc_SetSpeaker (int onoff)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   onoff &= 1;

   asm mov bx, 4
   asm mov ax, onoff
   asm call DWORD PTR voc_driver

   error = 0;
   return OK;
}

/* ----------------------------------------------- */
/* int voc_SetStatus (int *status);                */
/*                                                 */
/* Set the status word address.  The status of the */
/* current sample will always be written to this   */
/* location.  A value other than 0 indicates that  */
/* the sample is being processed.                  */
/*                                                 */
/* Return: ERROR or OK                             */
/* ----------------------------------------------- */

int voc_SetStatus (int *status)
{
   if (!installed)
   {
       error = VOCERR_NOT_INSTALLED;
       return ERROR;
   }

   asm mov bx, 5
   asm les di, status
   asm call DWORD PTR voc_driver

   error = 0;
   return OK;
}